#include "drawing.h"

GLUquadric* Drawing::quadric = 0;
GLuint Drawing::sphereDisplayList;
GLuint Drawing::circleDisplayList;
GLuint Drawing::cylinderDisplayList;

void Drawing::init () {
	#ifndef WIN32
	// In Linux, we don't need to recalculate everything.
	if (quadric) {
		return;
	}
	#endif
	
	quadric = gluNewQuadric();
	
	// Sphere
	sphereDisplayList = glGenLists(1);
	glNewList(sphereDisplayList, GL_COMPILE);
	gluSphere(quadric, 1, 32, 32);
	glEndList();
	
	// Cylinder
	cylinderDisplayList = glGenLists(1);
	glNewList(cylinderDisplayList, GL_COMPILE);
		glTranslatef(0, 0, 0.5f);
		gluDisk(quadric, 0, 1, 32, 1); // inner radius, outer radius, slices, loops
		glTranslatef(0, 0, -1);
		gluCylinder(quadric, 1, 1, 1, 32, 2); // base radius, top radius, height, slices, stacks
		glRotatef(180, 1, 0, 0);
		gluDisk(quadric, 0, 1, 32, 1); // inner radius, outer radius, slices, loops
	glEndList();
	
}

void Drawing::drawSphere (float x, float y, float z, float r) {
	glPushMatrix();
	
	glTranslatef(x, y, z);
	
	glScalef(r, r, r);
	
	glCallList(sphereDisplayList);
	
	glPopMatrix();
}

void Drawing::drawCylinder (float r, float l) {
	glPushMatrix();
	
	glScalef(r, r, l / 2);
	
	glCallList(cylinderDisplayList);
	
	glPopMatrix();
}

