#include "emitter.h"
#include "audio.h"
#include "drawing.h"
#include "globals.h"

#include <math.h>
#include <boost/foreach.hpp>

#include "SDL_opengl.h"

static const bool MOVING = true;

Emitter::Emitter (int i, float x, float z) : x(x), z(z), isDown(false), isPlaying(false) {
	if (MOVING) {
		vx = (rand() % 50) / 50.0f - 0.5f;
		vz = (rand() % 50) / 50.0f - 0.5f;
		
		vx *= 0.1f;
		vz *= 0.1f;
	} else {
		vx = 0;
		vz = 0;
	}
	
	audioData = Audio::audioData[i];
	
	alGenSources(1, &alSource);
	
    alSourcef(alSource, AL_PITCH, 1.0f);
    alSourcef(alSource, AL_GAIN, 1.0f);
    
    alSourcei(alSource, AL_BUFFER, Audio::buffers[i]);
    alSourcei(alSource, AL_LOOPING, AL_TRUE);
    
    alSource3f(alSource, AL_POSITION, x, 0, z);
    alSource3f(alSource, AL_VELOCITY, vx, 0, vz);
    
    if (rand() % 4 == 0) {
		alSourcePlay(alSource);
	}
	
	alSourcei(alSource, AL_LOOPING, AL_TRUE);
}

Emitter::~Emitter () {
	alDeleteSources(1, &alSource);
}

void Emitter::update () {
	bool wasDown = isDown;
	
	float dx = player.x - x;
	float dz = player.z - z;
	float dist = sqrt(dx*dx + dz*dz);
	
	isDown = (dist < 0.5f);
	
	ALint sourceState;
	alGetSourcei(alSource, AL_SOURCE_STATE, &sourceState);
	
	isPlaying = (sourceState == AL_PLAYING);
	
	/*if (isDown) {
		if (! isPlaying) {
			alSourcePlay(alSource);
		}
		
		alSourcei(alSource, AL_LOOPING, AL_TRUE);
	} else {
		alSourcei(alSource, AL_LOOPING, AL_FALSE);
	}*/
	
	if (isDown && ! wasDown) {
		if (isPlaying) {
			alSourceStop(alSource);
		} else {
			alSourcePlay(alSource);
		}
	}
	
	if (MOVING) {
		BOOST_FOREACH (Emitter* emitter, emitters) {
			if (emitter == this) { continue; }
			
			dx = emitter->x - x;
			dz = emitter->z - z;
			
			float distSq = dx*dx + dz*dz;
			
			vx -= 0.01f * dx / distSq;
			vz -= 0.01f * dz / distSq;
		}
		
		{
			dx = -10 - x;
			vx -= 0.1f / dx;
			
			dx = 10 - x;
			vx -= 0.1f / dx;
			
			dz = -10 - z;
			vz -= 0.1f / dz;
			
			dz = 10 - z;
			vz -= 0.1f / dz;
		}
		
		dx = vx;
		dz = vz;
		
		x += dx;
		z += dz;
		
		if (isDown) {
			player.x += dx;
			player.z += dz;
		}
	}
    
    alSource3f(alSource, AL_POSITION, x, 0, z);
    alSource3f(alSource, AL_VELOCITY, vx, 0, vz);
}

void Emitter::draw () {
	if (isPlaying) {
		glColor3f(0.2f, 0.6f, 0.2f);
	} else {
		glColor3f(0.7f, 0.1f, 0.1f);
	}
	
	float buttonHeight = (isDown) ? 0.05f : 0.1f;
	
	glPushMatrix();
	
	glTranslatef(x, buttonHeight / 2, z);
	
	glRotatef(90, 1, 0, 0);
	
	Drawing::drawCylinder(0.5f, buttonHeight);
	
	glPopMatrix();
}

