#include "floor.h"

#include <AL/al.h>

#include "SDL_opengl.h"

#include "stdlib.h"

#include "globals.h"

#include <boost/foreach.hpp>
#include <exception>
#include <iostream>
#include <math.h>

GLuint Floor::texture;


inline static short& get (short* source, int x, int z) {
	return source[x + z * Floor::SIZE];
}

inline float min (float a, float b) {
	return (a < b) ? a : b;
}

inline float max (float a, float b) {
	return (a < b) ? b : a;
}

void Floor::createTexture () {
	glGenTextures(1, &texture);
	glBindTexture(GL_TEXTURE_2D, texture);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
}

Floor::Floor () : frame(0) {
	
	/*
	textureData[0] = new short[SIZE*SIZE];
	textureData[1] = new short[SIZE*SIZE];
	*/
	
	for (int i = 0; i < SIZE*SIZE; i++) {
		textureData[0][i] = 0;
		textureData[1][i] = 0;
	}
}

Floor::~Floor () {
	/*delete textureData[0];
	delete textureData[1];*/
	
	//glDeleteTextures(1, &texture);
}

float distancePointSegment (float x1, float y1, float x2, float y2, float px, float py) {
	float dx = x2 - x1;
	float dy = y2 - y1;
	
	float lengthSq = dx * dx + dy * dy;
	
	float t = ((px - x1)*dx + (py - y1)*dy) / lengthSq;
	
	// If outside segment, clamp t (and therefore d) to the closest endpoint
    if (t < 0.0f) t = 0.0f;
    if (t > 1.0f) t = 1.0f;
    // Compute projected position from the clamped t
    
    float cx = x1 + t * dx;
    float cy = y1 + t * dy;
    
    dx = px - cx;
    dy = py - cy;
    
    return sqrt(dx*dx + dy*dy);
}

void Floor::update () {
	glBindTexture( GL_TEXTURE_2D, texture );
	
	short* source = textureData[(frame % 2 == 0) ? 0 : 1];
	short* dest   = textureData[(frame % 2 == 0) ? 1 : 0];
	
	frame++;
	
	BOOST_FOREACH(Emitter* emitter, emitters) {
		short value = 0;
		
		if (emitter->isPlaying) {
			//value = 32767;
			
			//value = sin(frame / 4.1415f) * 16384 + 16383;
			
			ALint buffer;
			
			alGetSourcei(emitter->alSource, AL_BUFFER, &buffer);
			
			ALint offset;
			
			alGetSourcei(emitter->alSource, AL_SAMPLE_OFFSET, &offset);
			
			ALint bufferLength;
			
			alGetBufferi(buffer, AL_SIZE, &bufferLength);
			
			bufferLength /= 2;
			
			short* data = (short*) emitter->audioData;
			
			int sum = 0;
			
			int i1 = max(0, offset - 500);
			int i2 = min(offset + 500, bufferLength);
			
			for (int i = i1; i < i2; i++) {
				sum += (data[i] >= 0) ? data[i] : -data[i];
			}
			
			value = min(5 * sum / (i2 - i1), 32767);
		}
		
		{
			float x = emitter->x;
			float z = emitter->z;
			float r = 0.5f;
			
			x = (x + 10) * SIZE / 20;
			z = (z + 10) * SIZE / 20;
			r = r * SIZE / 20;
			
			int ix1 = max(x - r, 0);
			int ix2 = min(x + r, SIZE);
			int iz1 = max(z - r, 0);
			int iz2 = min(z + r, SIZE);
			
			for (int ix = ix1; ix <= ix2; ix++)
			{
				for (int iz = iz1; iz <= iz2; iz++)
				{
					float dx = ix - x;
					float dz = iz - z;
					
					float rSq = r * r;
					
					if (dx * dx + dz * dz <= rSq) {
						get(source, ix, iz) = value;
						get(dest, ix, iz) = value;
					}
				}
			}
		}
	}
	
	// Player
	{
		float x2 = player.x;
		float z2 = player.z;
		
		float x1 = x2 - player.vx;
		float z1 = z2 - player.vz;
		
		float r = 0.15f;
		
		x1 = (x1 + 10) * SIZE / 20;
		z1 = (z1 + 10) * SIZE / 20;
		x2 = (x2 + 10) * SIZE / 20;
		z2 = (z2 + 10) * SIZE / 20;
		r = r * SIZE / 20;
		
		int ix1 = max(min(x1, x2) - r, 0);
		int ix2 = min(max(x1, x2) + r, SIZE);
		int iz1 = max(min(z1, z2) - r, 0);
		int iz2 = min(max(z1, z2) + r, SIZE);
		
		for (int ix = ix1; ix <= ix2; ix++)
		{
			for (int iz = iz1; iz <= iz2; iz++)
			{
				float dist = distancePointSegment(x1, z1, x2, z2, ix, iz);
				
				if (dist <= r) {
					float maxValue = (dist / r - 0.5f) * 32767;
					
					if (dist <= r / 2) {
						maxValue = 0;
					}
					
					get(source, ix, iz) = min(maxValue, get(source, ix, iz));
					get(dest, ix, iz) = min(maxValue, get(dest, ix, iz));
				}
			}
		}
	}
	
	for (int x = 1; x < SIZE - 1; x++)
	{
		for (int z = 1; z < SIZE - 1; z++)
		{
			get(dest, x, z) = (
					(get(source, x-1, z) +
					 get(source, x+1, z) +
					 get(source, x, z-1) +
					 get(source, x, z+1)) >> 1
				) - get(dest, x, z);
			
			if (get(dest, x, z) < 0) {
				get(dest, x, z) = 0;
			}
			
			// This enables dampening
			get(dest, x, z) -= (get(dest, x, z) >> 10);
		}
	}

	glTexImage2D(GL_TEXTURE_2D, 0, GL_LUMINANCE, SIZE, SIZE, 0, GL_LUMINANCE, GL_SHORT, dest);
}

void Floor::draw () {
	glColor3f(1, 1, 1);
	
	glDisable(GL_LIGHTING);
	
	glEnable(GL_TEXTURE_2D);
	
	glBindTexture( GL_TEXTURE_2D, texture );
	
	glNormal3f(0, 1, 0);
	
	glBegin(GL_QUADS);
		glTexCoord2i( 0, 0 );
		glVertex3f(-10, 0, -10);
		
		glTexCoord2i( 0, 1 );
		glVertex3f(-10, 0, 10);
		
		glTexCoord2i( 1, 1 );
		glVertex3f(10, 0, 10);
		
		glTexCoord2i( 1, 0 );
		glVertex3f(10, 0, -10);
	glEnd();
	
	glDisable(GL_TEXTURE_2D);
	
	glEnable(GL_LIGHTING);
	
}


